<?php

namespace Drupal\structure_map;

use Drupal\Core\Config\ConfigManagerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Field\FieldTypePluginManagerInterface;
use Drupal\field\Entity\FieldConfig;
use Drupal\user\Entity\Role;

/**
 * Class to get information about an entity type.
 */
class EntityTypeInfo {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The config manager.
   *
   * @var \Drupal\Core\Config\ConfigManagerInterface
   */
  protected $configManager;

  /**
   * The entity display repository.
   *
   * @var \Drupal\Core\Entity\EntityDisplayRepositoryInterface
   */
  protected $entityDisplayRepository;

  /**
   * The entity form display storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $formDisplayStorage;

  /**
   * The entity field manager.
   *
   * @var \Drupal\Core\Entity\EntityFieldManagerInterface
   */
  protected $entityFieldManager;

  /**
   * The field widget plugin manager.
   *
   * @var \Drupal\Core\Plugin\DefaultPluginManager
   */
  protected $widgetPluginManager;

  /**
   * The field type plugin manager.
   *
   * @var \Drupal\Core\Field\FieldTypePluginManagerInterface
   */
  protected $fieldTypeManager;

  /**
   * Constructs a new EntityTypeInfo object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Config\ConfigManagerInterface $config_manager
   *   The config manager.
   * @param \Drupal\Core\Field\FieldTypePluginManagerInterface $field_type_manager
   *   The field type plugin manager.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, ConfigManagerInterface $config_manager, FieldTypePluginManagerInterface $field_type_manager) {
    $this->entityTypeManager = $entity_type_manager;
    $this->configManager = $config_manager;
    $this->entityDisplayRepository = \Drupal::service('entity_display.repository');
    $this->formDisplayStorage = $entity_type_manager->getStorage('entity_form_display');
    $this->entityFieldManager = \Drupal::service('entity_field.manager');
    $this->widgetPluginManager = \Drupal::service('plugin.manager.field.widget');
    $this->fieldTypeManager = $field_type_manager;
  }

  /**
   * Retrieves the basic information of an entity type and bundle.
   *
   * @param string $entity_type
   *   The entity type machine name.
   * @param string $bundle
   *   The bundle machine name.
   *
   * @return array
   *   An array containing the basic information.
   */
  public function getBasicEntityInfo(string $entity_type, string $bundle): array {
    $entity_type_definition = $this->entityTypeManager->getDefinition($entity_type);
    $bundle_name = \Drupal::service('entity_type.bundle.info')->getBundleInfo($entity_type)[$bundle]['label'];
    $bundle_machine_name = $bundle;
    // Entity and bundle names.
    $entity_name = $entity_type_definition->getLabel()->__toString();
    $entity_machine_name = $entity_type_definition->id();

    // Title field machine name.
    $title_field_machine_name = $entity_type_definition->get('entity_keys')['label'];

    // Get the base field definitions for the entity type.
    $base_field_definitions = \Drupal::service('entity_field.manager')->getBaseFieldDefinitions($entity_type);

    // Retrieve the human-readable label for the title field.
    $title_field_label = '';
    if (isset($base_field_definitions[$title_field_machine_name])) {
      $title_field_label = $base_field_definitions[$title_field_machine_name]->getLabel()->__toString();
    }

    // Check which roles have edit and view access to this entity.
    $roles = Role::loadMultiple();
    $edit_any_access_roles = [];
    $edit_own_access_roles = [];

    // Normalize entity type for nodes.
    if ($entity_type === 'node') {
      $entity_type = 'content';
    }

    foreach ($roles as $role) {
      $role_permissions = \Drupal::service('user.permissions')->getPermissions($role->id());
      $edit_permission_any = 'edit any ' . $bundle . ' ' . $entity_type;
      $edit_permission_own = 'edit own ' . $bundle . ' ' . $entity_type;

      if ($role->hasPermission($edit_permission_any)) {
        $edit_any_access_roles[] = $role->label();
      }
      if ($role->hasPermission($edit_permission_own)) {
        $edit_own_access_roles[] = $role->label();
      }
    }

    return [
      'entity_name' => $entity_name,
      'entity_machine_name' => $entity_machine_name,
      'bundle_name' => $bundle_name,
      'bundle_machine_name' => $bundle_machine_name,
      'title_field_label' => $title_field_label,
      'edit_own_access' => $edit_own_access_roles,
      'edit_any_access' => $edit_any_access_roles,
    ];
  }

  /**
   * Retrieves the fields information of an entity type and bundle.
   *
   * @param string $entity_type
   *   The entity type machine name.
   * @param string $bundle
   *   The bundle machine name.
   *
   * @return array
   *   An array containing the fields information.
   */
  public function getBasicFieldEntityInfo(string $entity_type, string $bundle): array {
    $fields_info = [];
    $field_definitions = $this->entityFieldManager->getFieldDefinitions($entity_type, $bundle);

    foreach ($field_definitions as $field_name => $field_definition) {

      // Skip base fields and only include configurable fields.
      if ($field_definition instanceof FieldConfig) {

        // Load the field summary.
        $field_config = FieldConfig::loadByName($entity_type, $bundle, $field_name);
        /** @var \Drupal\field\FieldConfigInterface $field_config */
        $field_storage = $field_config->getFieldStorageDefinition();
        $storage_summary = $this->fieldTypeManager->getStorageSettingsSummary($field_storage);
        $instance_summary = $this->fieldTypeManager->getFieldSettingsSummary($field_config);
        $summary_list = [...$storage_summary, ...$instance_summary];
        $field_type_summary = implode('<br/>', $summary_list);

        $label = $field_definition->getLabel();
        $field_info = [
          'label' => $label,
          'machine_name' => $field_name,
          'field_type' => $this->fieldTypeManager->getDefinitions()[$field_storage->getType()]['label'],
          'required' => $field_definition->isRequired() ? 'Yes' : 'No',
          'cardinality' => $field_definition->getFieldStorageDefinition()->getCardinality(),
          'default_value' => $field_definition->getDefaultValueLiteral(),
          'settings' => $field_type_summary,
        ];
        $fields_info[] = $field_info;
      }
    }

    return $fields_info;
  }

  /**
   * Retrieves the form display information of an entity type and bundle.
   *
   * @param string $entity_type
   *   The entity type machine name.
   * @param string $bundle
   *   The bundle machine name.
   *
   * @return array
   *   An array containing the form display information.
   */
  public function getFormDisplayInfo(string $entity_type, string $bundle): array {
    $entity_display_repository = $this->entityDisplayRepository;
    $form_display_storage = $this->formDisplayStorage;
    $field_definitions = $this->entityFieldManager->getFieldDefinitions($entity_type, $bundle);
    $widget_plugin_manager = $this->widgetPluginManager;

    // Create an array with sto store the information for the different modes.
    $form_display_info = [];

    // Get the form modes.
    $form_modes = $entity_display_repository->getFormModes($entity_type);
    // Add the default form mode to the form modes array.
    $form_modes['default'] = ['label' => 'Default'];

    foreach ($form_modes as $form_mode => $form_mode_info) {

      // Load the form display for the current form mode.
      $form_display = $form_display_storage->load($entity_type . '.' . $bundle . '.' . $form_mode);
      if (!$form_display) {
        continue;
      }

      // Set the form mode label.
      $form_display_info[$form_mode]['label'] = $form_mode_info['label'];

      if ($form_display) {
        // Get a list of hidden fields.
        $hidden_fields = [];
        foreach ((array) $form_display as $key => $value) {
          if (strpos($key, 'hidden') !== FALSE) {
            $hidden_fields = array_keys($value);
          }
        }

        // Get the disabled fields.
        foreach ($hidden_fields as $key => $field_name) {
          // Get the field label.
          $field_label = $field_definitions[$field_name] ? $field_definitions[$field_name]->getLabel() : $field_name;

          $form_display_info[$form_mode]['disabled'][] = [
            'field_label' => $field_label,
            'machine_name' => $field_name,
          ];
        }

        // Get the enabled fields.
        foreach ($form_display->getComponents() as $field_name => $component) {
          // Get the field label.
          $field_label = $field_definitions[$field_name] ? $field_definitions[$field_name]->getLabel() : $field_name;

          // Get the widget name.
          $widget = $component['type'];

          // Check if the widget type is valid.
          if (!empty($widget)) {
            // Get the human-readable name of the widget.
            $widget_definition = $widget_plugin_manager->getDefinition($widget);
            $widget_label = $widget_definition['label'];

            // Get the widget summary.
            $plugin = $form_display->getRenderer($field_name);
            if ($plugin) {
              $widget_settings = $plugin->settingsSummary();
              $widget_settings_summary = implode('<br/>', $widget_settings);
            }
            else {
              $widget_settings_summary = '';
            }
          }

          // Get the field group if exists.
          $field_group = $component['region'] ?? NULL;

          $form_display_info[$form_mode]['enabled'][] = [
            'field_label' => $field_label,
            'machine_name' => $field_name,
            'widget_name' => $widget,
            'widget_label' => $widget_label,
            'widget_configuration' => $widget_settings_summary,
            'field_group' => $field_group,
          ];
        }
      }
    }
    return $form_display_info;
  }

  /**
   * Retrieves the view display information of an entity type and bundle.
   *
   * @param string $entity_type
   *   The entity type machine name.
   * @param string $bundle
   *   The bundle machine name.
   *
   * @return array
   *   An array containing the form display information.
   */
  public function getViewDisplayInfo(string $entity_type, string $bundle): array {
    $entity_display_repository = $this->entityDisplayRepository;
    $view_display_storage = $this->entityTypeManager->getStorage('entity_view_display');
    $field_definitions = $this->entityFieldManager->getFieldDefinitions($entity_type, $bundle);
    $widget_plugin_manager = $this->widgetPluginManager;

    // Create an array to store the information for the different modes.
    $view_display_info = [];

    // Get the view modes.
    $view_modes = $entity_display_repository->getViewModes($entity_type);
    // Add the default view mode to the view modes array.
    $view_modes['default'] = ['label' => 'Default'];

    foreach ($view_modes as $view_mode => $view_mode_info) {

      // Load the view display for the current view mode.
      $view_display = $view_display_storage->load($entity_type . '.' . $bundle . '.' . $view_mode);
      if (!$view_display) {
        continue;
      }

      // Set the view mode label.
      $view_display_info[$view_mode]['label'] = $view_mode_info['label'];

      if ($view_display) {
        // Get a list of hidden fields.
        $hidden_fields = [];
        foreach ((array) $view_display as $key => $value) {
          if (strpos($key, 'hidden') !== FALSE) {
            $hidden_fields = array_keys($value);
          }
        }

        // Get the disabled fields.
        foreach ($hidden_fields as $field_name) {
          // Get the field label.
          $field_label = $field_definitions[$field_name] ? $field_definitions[$field_name]->getLabel() : $field_name;

          $view_display_info[$view_mode]['disabled'][] = [
            'field_label' => $field_label,
            'machine_name' => $field_name,
          ];
        }

        // Get the enabled fields.
        foreach ($view_display->getComponents() as $field_name => $component) {
          // Get the field label.
          $field_label = $field_definitions[$field_name] ? $field_definitions[$field_name]->getLabel() : $field_name;

          // Get the label settings.
          $label_settings = $component['label'];

          // Get the format and format settings.
          $format = $component['type'];
          $format_settings = $component['settings'];

          // Get the widget summary.
          $plugin = $view_display->getRenderer($field_name);
          if ($plugin) {
            $widget_settings = $plugin->settingsSummary();
            $widget_settings_summary = implode('<br/>', $widget_settings);
          }
          else {
            $widget_settings_summary = '';
          }

          $view_display_info[$view_mode]['enabled'][] = [
            'field_label' => $field_label,
            'machine_name' => $field_name,
            'label_settings' => $label_settings,
            'format' => $format,
            'format_settings' => $widget_settings_summary,
          ];
        }
      }
    }
    return $view_display_info;
  }

  /**
   * Helper function to format an array to text.
   *
   * @param array $array
   *   The array to format.
   * @param int $level
   *   The level of nesting.
   *
   * @return string
   *   The formatted text.
   */
  public function formatArrayToTextHelper($array, $level = 0) {
    $formattedText = '';
    foreach ($array as $key => $value) {
      // Add indentation based on the level of nesting.
      $indentation = str_repeat('&nbsp;&nbsp;', $level);
      // Format the current key-value pair.
      if (is_array($value)) {
        // If the value is an array, recursively format it.
        $formattedText .= $indentation . $key . ": <br/>\n" . $this->formatArrayToTextHelper($value, $level + 1);
      }
      else {
        // Format the value, handling boolean values explicitly.
        $formattedText .= $indentation . $key . ': ' . (is_bool($value) ? ($value ? 'true' : 'false') : $value) . " <br/>\n";
      }
    }
    return $formattedText;
  }

}
